@@ -79,7 +79,13 @@
 //!   detection. This includes `cargo build` of library crates and `cargo check`
 //!   of binary and library crates.
 //!
-//! - The attribute is useless in code built with `panic = "abort"`.
+//! - The attribute is useless in code built with `panic = "abort"`. Code must
+//!   be built with `panic = "unwind"` (the default) in order for any panics to
+//!   be detected. After confirming absence of panics, you can of course still
+//!   ship your software as a `panic = "abort"` build.
+//!
+//! - Const functions are not supported. The attribute will fail to compile if
+//!   placed on a `const fn`.
 //!
 //! If you find that code requires optimization to pass `#[no_panic]`, either
 //! make no-panic an optional dependency that you only enable in release builds,
@@ -126,11 +132,12 @@
 //! [Kixunil]: https://github.com/Kixunil
 //! [`dont_panic`]: https://github.com/Kixunil/dont_panic
 
-#![doc(html_root_url = "https://docs.rs/no-panic/0.1.32")]
+#![doc(html_root_url = "https://docs.rs/no-panic/0.1.36")]
 #![allow(
     clippy::doc_markdown,
     clippy::match_same_arms,
-    clippy::missing_panics_doc
+    clippy::missing_panics_doc,
+    clippy::uninlined_format_args
 )]
 #![cfg_attr(all(test, exhaustive), feature(non_exhaustive_omitted_patterns_lint))]
 
@@ -139,6 +146,7 @@ extern crate proc_macro;
 use proc_macro::TokenStream;
 use proc_macro2::{Span, TokenStream as TokenStream2};
 use quote::quote;
+use std::mem;
 use syn::parse::{Error, Nothing, Result};
 use syn::{
     parse_quote, FnArg, GenericArgument, Ident, ItemFn, Pat, PatType, Path, PathArguments,
@@ -149,19 +157,36 @@ use syn::{
 pub fn no_panic(args: TokenStream, input: TokenStream) -> TokenStream {
     let args = TokenStream2::from(args);
     let input = TokenStream2::from(input);
-    let expanded = match parse(args, input.clone()) {
-        Ok(function) => expand_no_panic(function),
+    TokenStream::from(match parse(args, input.clone()) {
+        Ok(function) => {
+            let expanded = expand_no_panic(function);
+            quote! {
+                #[cfg(not(doc))]
+                #expanded
+                // Keep generated parameter names out of doc builds.
+                #[cfg(doc)]
+                #input
+            }
+        }
         Err(parse_error) => {
             let compile_error = parse_error.to_compile_error();
-            quote!(#compile_error #input)
+            quote! {
+                #compile_error
+                #input
+            }
         }
-    };
-    TokenStream::from(expanded)
+    })
 }
 
 fn parse(args: TokenStream2, input: TokenStream2) -> Result<ItemFn> {
     let function: ItemFn = syn::parse2(input)?;
     let _: Nothing = syn::parse2::<Nothing>(args)?;
+    if function.sig.constness.is_some() {
+        return Err(Error::new(
+            Span::call_site(),
+            "no_panic attribute on const fn is not supported",
+        ));
+    }
     if function.sig.asyncness.is_some() {
         return Err(Error::new(
             Span::call_site(),
@@ -214,20 +239,25 @@ fn make_impl_trait_wild_in_path(path: &mut Path) {
 
 fn expand_no_panic(mut function: ItemFn) -> TokenStream2 {
     let mut move_self = None;
+    let mut arg_attrs = Vec::new();
     let mut arg_pat = Vec::new();
     let mut arg_val = Vec::new();
     for (i, input) in function.sig.inputs.iter_mut().enumerate() {
-        let numbered = Ident::new(&format!("__arg{}", i), Span::call_site());
         match input {
-            FnArg::Typed(PatType { pat, .. })
+            FnArg::Typed(PatType { attrs, pat, .. })
                 if match pat.as_ref() {
                     Pat::Ident(pat) => pat.ident != "self",
                     _ => true,
                 } =>
             {
-                arg_pat.push(quote!(#pat));
-                arg_val.push(quote!(#numbered));
-                *pat = parse_quote!(mut #numbered);
+                let arg_name = if let Pat::Ident(original_name) = &**pat {
+                    original_name.ident.clone()
+                } else {
+                    Ident::new(&format!("__arg{}", i), Span::call_site())
+                };
+                arg_attrs.push(attrs);
+                arg_pat.push(mem::replace(&mut *pat, parse_quote!(mut #arg_name)));
+                arg_val.push(arg_name);
             }
             FnArg::Typed(_) | FnArg::Receiver(_) => {
                 move_self = Some(quote! {
@@ -269,7 +299,7 @@ fn expand_no_panic(mut function: ItemFn) -> TokenStream2 {
     } else {
         Some(Token![unsafe](Span::call_site()))
     };
-    function.block = Box::new(parse_quote!({
+    *function.block = parse_quote!({
         struct __NoPanic;
         #unsafe_extern extern "C" {
             #[link_name = #message]
@@ -286,13 +316,14 @@ fn expand_no_panic(mut function: ItemFn) -> TokenStream2 {
         let __result = (move || #ret {
             #move_self
             #(
+                #(#arg_attrs)*
                 let #arg_pat = #arg_val;
             )*
             #(#stmts)*
         })();
         ::core::mem::forget(__guard);
         __result
-    }));
+    });
 
     quote!(#function)
 }
